from src.tf2_sku import to_sku, from_sku

from unittest import TestCase


MANN_CO_KEY = {
    "defindex": 5021,
    "quality": 6,
    "effect": -1,
    "australium": False,
    "craftable": True,
    "wear": -1,
    "skin": -1,
    "strange": False,
    "killstreak_tier": -1,
    "target_defindex": -1,
    "festivized": False,
    "craft_number": -1,
    "crate_number": -1,
    "output_defindex": -1,
    "output_quality": -1,
    "paint": -1,
}


class TestSKU(TestCase):
    def test_key(self):
        sku = "5021;6"
        item = {"defindex": 5021, "quality": 6}
        result = to_sku(item)

        item_dict = from_sku(sku)

        skued = to_sku(MANN_CO_KEY)

        self.assertEqual(result, sku)
        self.assertEqual(item_dict, MANN_CO_KEY)
        self.assertEqual(sku, skued)

    def test_from_sku(self):
        sku = "5021;6"
        item = from_sku(sku)

        self.assertEqual(item, MANN_CO_KEY)

    def test_ellis_cap_double(self):
        item = {"defindex": 236, "quality": 6}
        sku = to_sku(item)

        item_again = from_sku(sku)
        sku_again = to_sku(item_again)

        self.assertEqual(sku, sku_again)

    def test_skin(self):
        original = "199;5;u702;w3;pk292;strange;kt-3"
        item = from_sku(original)
        sku = to_sku(item)

        self.assertEqual(sku, original)

    def test_types(self):
        item_dict = from_sku("5021;6")
        sku = to_sku(item_dict)

        self.assertTrue(isinstance(item_dict, dict))
        self.assertTrue(isinstance(sku, str))



from dataclasses import dataclass, fields


MAPPING = {
    "effect": "u{}",
    "australium": "australium",
    "craftable": "uncraftable",
    "wear": "w{}",
    "skin": "pk{}",
    "strange": "strange",
    "killstreak_tier": "kt-{}",
    "target_defindex": "td-{}",
    "festivized": "festive",
    "craft_number": "n{}",
    "crate_number": "c{}",
    "output_defindex": "od-{}",
    "output_quality": "oq-{}",
    "paint": "p",
    # "sheen": "ks-{}",
    # "killstreaker": "ke-{}",
}


@dataclass
class SKU:
    defindex: int
    quality: int
    effect: int = -1
    australium: bool = False
    craftable: bool = True
    wear: int = -1
    skin: int = -1
    strange: bool = False
    killstreak_tier: int = -1
    target_defindex: int = -1
    festivized: bool = False
    craft_number: int = -1
    crate_number: int = -1
    output_defindex: int = -1
    output_quality: int = -1
    paint: int = -1
    # sheen: int = -1
    # killstreaker: int = -1

    def __str__(self) -> str:
        sku = f"{self.defindex};{self.quality}"

        for field in fields(self):
            if field.name in ["defindex", "quality"]:
                continue

            value = getattr(self, field.name)

            if value != field.default:
                sku += ";" + MAPPING[field.name].format(value)

        return sku



from .utils import SKU, MAPPING


def to_sku(item: dict) -> str:
    """Takes an item dictionary and formats it to a SKU.

    :param item: Item dictionary containing specific keys and values
    :type item: dict

    :return: SKU string
    :rtype: str

    Example:
    to_sku({
        "defindex": 199,
        "quality": 5,
        "effect": 702,
        "australium": False,
        "craftable": True,
        "wear": 3,
        "skin": 292,
        "strange": True,
        "killstreak_tier": 3,
        "target_defindex": -1,
        "festivized": False,
        "craft_number": -1,
        "crate_number": -1,
        "output_defindex": -1,
        "output_quality": -1,
        "paint": -1,
    })
    """
    sku = SKU(**item)
    return str(sku)


def from_sku(sku: str) -> dict:
    """Takes a SKU and formats it to an item dictionary.

    :param sku: SKU string
    :type sku: str

    :return: Item dictionary containing specific keys and values
    :rtype: dict

    Example:
    from_sku("199;5;u702;w3;pk292;strange;kt-3")
    """
    parts = sku.split(";")
    item = {"defindex": int(parts[0]), "quality": int(parts[1])}

    for part in parts[2:]:
        for key in MAPPING:
            default = MAPPING[key].format("")

            if not default in part:
                continue

            value = part.replace(default, "")

            if key == "craftable":
                value = False

            else:
                if not value:
                    value = True

                else:
                    value = int(value)

            item[key] = value
            break

    sku = SKU(**item)
    return sku.__dict__



"""
tf2-sku
Format TF2 items as strings or objects.
"""

__title__ = "tf2-sku"
__author__ = "offish"
__license__ = "MIT"
__version__ = "2.0.0"

from .sku import to_sku, from_sku
from .utils import SKU, MAPPING
